/*
 * ffsox_ar.c
 * Copyright (C) 2011, 2012 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdio.h>
#include "ffsox.h"

ffsox_ar_t *ffsox_ar_init(ffsox_ar_t *ar, pb_str_t *fn1, int force_stereo)
{
  if (NULL==ffi_init(&ar->ffi,fn1,1,force_stereo))
    return NULL;

  if (NULL==(ar->frame=avcodec_alloc_frame()))
    return NULL;

  ar->state=FFSOX_AR_RUN;
  ffsox_ar_next(ar);

  return ar;
}

ffsox_ar_t *ffsox_ar_cleanup(ffsox_ar_t *ar)
{
  if (NULL!=ar->frame)
    av_free(ar->frame);

  ffi_cleanup(&ar->ffi);

  return ar;
}

void ffsox_ar_next(ffsox_ar_t *ar)
{
  ffi_st_t *ast=ar->ffi.ast;
  AVFormatContext *ic=ar->ffi.ic;
  AVCodecContext *dc=ast->dc;
  int got_frame;

  avcodec_get_frame_defaults(ar->frame);

  while (FFSOX_AR_RUN==ar->state) {
    got_frame=0;

    if (0!=av_read_frame(ic,&ar->pkt)) {
      memset(&ar->pkt,0,sizeof ar->pkt);
      ar->state=FFSOX_AR_FLUSH;
      break;
    }

    if (ar->pkt.stream_index!=ast->index)
      goto cleanup;
    else if (avcodec_decode_audio4(dc,ar->frame,&got_frame,&ar->pkt)<0) {
      ar->state=FFSOX_AR_ERROR;
      goto cleanup;
    }
  cleanup:
    av_free_packet(&ar->pkt);

    if (got_frame)
      return;
  }

  if (FFSOX_AR_FLUSH==ar->state) {
    if (avcodec_decode_audio4(dc,ar->frame,&got_frame,&ar->pkt)<0)
      ar->state=FFSOX_AR_ERROR;
    else if (!got_frame)
      ar->state=FFSOX_AR_DONE;
  }
}
